@@ -37,19 +37,18 @@ from .utils import get_sample_writer
 
 from SwissArmyTransformer import mpu
 from SwissArmyTransformer.data_utils import make_loaders
-from SwissArmyTransformer.tokenization import get_tokenizer
+from SwissArmyTransformer.ops import LayerNorm
+from SwissArmyTransformer.arguments import set_random_seed, initialize_distributed
 
-
-def training_main(args, model_cls, forward_step_function, create_dataset_function, init_function=None):
+def training_main(args, model_cls, forward_step_function, create_dataset_function, handle_metrics=None, init_function=None):
     """Main training program."""
     hooks = {
         'forward_step': forward_step_function,
         'init_function': init_function,
-        'create_dataset_function': create_dataset_function
+        'create_dataset_function': create_dataset_function,
+        'handle_metrics': handle_metrics,
     }
 
-    torch.backends.cuda.matmul.allow_tf32 = False
-    torch.backends.cudnn.enabled = False  # Disable CuDNN.
     timers = Timers()  # Timer.
 
     # Experiment Name
@@ -58,19 +57,29 @@ def training_main(args, model_cls, forward_step_function, create_dataset_functio
     else:
         args.experiment_name = args.experiment_name + datetime.now().strftime("%m-%d-%H-%M")
 
-    # Pytorch distributed. must before seed
-    initialize_distributed(args)
-    set_random_seed(args.seed)  # Random seeds for reproducability.
-    # init tokenizer
-    get_tokenizer(args)  # set args.vocab_size.
+    # Pytorch distributed. must before seed. ALREADY MOVED TO arguments.py!
+    # if isinstance(model_cls, type):
+    #     initialize_distributed(args)
+    #     set_random_seed(args.seed)  # Random seeds for reproducability.
+
     # Data stuff.
     train_data, val_data, test_data = make_loaders(args, hooks['create_dataset_function'])
+    if args.epochs:
+        args.train_iters = len(train_data)
+        if args.eval_interval is None:
+            args.eval_interval = len(train_data)//args.epochs
+        if args.save_interval is None:
+            args.save_interval = len(train_data)//args.epochs
 
-    # Model, optimizer, and learning rate.
-    model, optimizer = setup_model_and_optimizer(args, model_cls)
+    # Build model
+    if isinstance(model_cls, type):
+        model = get_model(args, model_cls)
+        
+    else:
+        model = model_cls
 
     # Config model IO
-    if args.load is not None:
+    if args.load is not None and isinstance(model_cls, type): # If you pass a model by yourself, you need to load it by youself as well.
         args.iteration = load_checkpoint(model, args)
         # if we don't load optim_states, filelock is no more needed.
         # with FileLock("/root/checkpoint_lock", timeout=-1):
@@ -81,6 +90,13 @@ def training_main(args, model_cls, forward_step_function, create_dataset_functio
         args.save = os.path.join(args.save, args.experiment_name)
     torch.distributed.barrier()
 
+    # init hook before building deepspeed model and optimizer
+    if hooks['init_function'] is not None:
+        hooks['init_function'](args, model)
+
+    # Optimization related things
+    model, optimizer = setup_model_untrainable_params_and_optimizer(args, model)
+
     # initialize lr scheduler
     lr_scheduler = get_learning_rate_scheduler(optimizer, args.iteration, args)
 
@@ -101,10 +117,6 @@ def training_main(args, model_cls, forward_step_function, create_dataset_functio
             start_iter_val = (args.train_iters // args.save_interval) * args.eval_interval
             val_data.batch_sampler.start_iter = start_iter_val % len(val_data)
 
-    # init hook before training
-    if hooks['init_function'] is not None:
-        hooks['init_function'](args, model, optimizer)
-
     # training 
     iteration = 0
     if args.train_iters > 0:
@@ -128,7 +140,7 @@ def training_main(args, model_cls, forward_step_function, create_dataset_functio
     if args.do_test and test_data is not None:
         prefix = 'the end of training for test data'
         evaluate_and_print_results(prefix, iter(test_data),
-            model, len(test_data) if args.strict_eval else args.eval_iters, args, timers, True, hooks=hooks)
+            model, len(test_data) if args.strict_eval else args.eval_iters, args, timers, True, split='test', hooks=hooks)
 
 
 def get_model(args, model_cls):
@@ -151,12 +163,11 @@ def get_model(args, model_cls):
     return model
 
 
-def setup_model_and_optimizer(args, model_cls, config_params=None):
+def setup_model_untrainable_params_and_optimizer(args, model, config_params=None):
     """Setup model and optimizer."""
 
-    model = get_model(args, model_cls)
-
-    model.disable_untrainable_params()  # mark trainable params
+    if hasattr(model, 'disable_untrainable_params'):
+        model.disable_untrainable_params() # mark trainable params
 
     param_groups = get_optimizer_param_groups(model)
 
@@ -169,7 +180,7 @@ def setup_model_and_optimizer(args, model_cls, config_params=None):
                 args=args,
                 mpu=mpu,
                 dist_init_required=False,
-                config_params=config_params
+                config_params=args.deepspeed_config
             )
         else:
             raise ValueError('Currently, we only support training with deepspeed.')
@@ -183,7 +194,7 @@ def get_params_for_weight_decay_optimization(module):
     weight_decay_params = {'params': []}
     no_weight_decay_params = {'params': [], 'weight_decay': 0.0}
     for module_ in module.modules():
-        if isinstance(module_, (mpu.LayerNorm, torch.nn.LayerNorm)):
+        if isinstance(module_, (LayerNorm, torch.nn.LayerNorm)):
             no_weight_decay_params['params'].extend(
                 [p for p in list(module_._parameters.values())
                  if p is not None and p.requires_grad])
@@ -263,7 +274,6 @@ def train(model, optimizer, lr_scheduler,
     # Turn on training mode which enables dropout.
     model.train()
     
-
     # Tracking loss.
     total_lm_loss = 0.0
     total_metrics = defaultdict(float)
@@ -286,7 +296,9 @@ def train(model, optimizer, lr_scheduler,
         # Update losses.
         total_lm_loss += lm_loss.data.detach().float()
         for name in metrics:
-            total_metrics[name] += metrics[name].data.detach().float().item()
+            if not 'eval' in name:
+                assert len(metrics[name].shape)==0, 'metrics without eval must be scalar'
+                total_metrics[name] += metrics[name].data.detach().float().item()
 
         # Logging.
         if args.iteration % args.log_interval == 0:
@@ -323,7 +335,7 @@ def train(model, optimizer, lr_scheduler,
                 eval_iters = args.eval_iters
             prefix = 'iteration {}'.format(args.iteration)
             evaluate_and_print_results(
-                prefix, val_data_iterator, model, eval_iters, args, timers, False, step=args.iteration, summary_writer=summary_writer, hooks=hooks)
+                prefix, val_data_iterator, model, eval_iters, args, timers, False, step=args.iteration, split='val', summary_writer=summary_writer, hooks=hooks)
 
         if args.exit_interval and args.iteration % args.exit_interval == 0:
             torch.distributed.barrier()
@@ -358,10 +370,11 @@ def train_step(data_iterator, model, optimizer, lr_scheduler,
 
         loss_checker = lm_loss_reduced
         for name in metrics:
-            metrics[name] = metrics[name].detach().clone()
-            torch.distributed.all_reduce(metrics[name].data)
-            metrics[name].data /= args.world_size
-            loss_checker = loss_checker + metrics[name]
+            if not 'eval' in name:
+                metrics[name] = metrics[name].detach().clone()
+                torch.distributed.all_reduce(metrics[name].data)
+                metrics[name].data /= args.world_size
+                loss_checker = loss_checker + metrics[name]
         if loss_checker.isnan().any() or loss_checker.isinf().any():
             print('Skipping backward and optimizer step for nan or inf in forwarding metrics/loss!')
             return lm_loss.detach(), 1, metrics
@@ -416,20 +429,29 @@ def backward_step(optimizer, model, loss, args, timers):
 
     return
 
-def evaluate(data_iterator, model, eval_iters, args, timers, verbose=False, hooks={}):
+def evaluate(data_iterator, model, eval_iters, args, timers, split, verbose=False, has_last=True, hooks={}):
     """Evaluation."""
     forward_step = hooks['forward_step']
     # Turn on evaluation mode which disables dropout.
     model.eval()
-
+    rank = torch.distributed.get_rank(group=mpu.get_data_parallel_group())
     total_lm_loss, metrics_total = 0, {}
+    if split=='val':
+        last_shape = args.val_last_shape
+        drop_number = args.val_drop_number
+    else:
+        assert split=='test'
+        last_shape = args.test_last_shape
+        drop_number = args.test_drop_number
+    is_scalar = {}
     with torch.no_grad():
         iteration = 0
         while iteration < eval_iters:
-            iteration += 1 
+            iteration += 1
             if verbose and iteration % args.log_interval == 0:
                 print_rank_0('Evaluating iter {}/{}'.format(iteration, eval_iters))
             # Forward evaluation.
+            # try:
             lm_loss, metrics = forward_step(data_iterator, model, args, timers)
             '''when contiguous memory optimizations are enabled, the buffers
             allocated by the optimizations are deallocated during backward pass
@@ -438,35 +460,58 @@ def evaluate(data_iterator, model, eval_iters, args, timers, verbose=False, hook
             if args.deepspeed and args.deepspeed_activation_checkpointing:
                 deepspeed.checkpointing.reset()
             total_lm_loss += lm_loss.data.detach().float().item()
+            is_last = True if iteration == eval_iters and args.strict_eval and len(last_shape)>0 else False
             for name in metrics:
                 if name not in metrics_total:
-                    metrics_total[name] = 0.0
-                metrics_total[name] += metrics[name]
+                    metrics_total[name] = []
+                is_scalar[name] = True if len(metrics[name].shape)==0 else False
+                shape = list(metrics[name].shape)
+                if not is_scalar[name] and is_last and metrics[name].shape[0] != last_shape[0]:
+                    # pad tensor's first dim to args.batch_size
+                    metrics[name] = torch.concat([metrics[name], torch.zeros([last_shape[0]-metrics[name].shape[0]] + shape[1:], dtype=metrics[name].dtype, device=metrics[name].device)])
+                if rank==0:
+                    metrics_gathered = [torch.zeros_like(metrics[name], dtype=metrics[name].dtype, device=metrics[name].device) for _ in range(args.world_size)]
+                else:
+                    # metrics_gathered = None
+                    metrics_gathered = [torch.zeros_like(metrics[name], dtype=metrics[name].dtype, device=metrics[name].device) for _ in range(args.world_size)]
+                # torch.distributed.gather(metrics[name], metrics_gathered, 0)
+                torch.distributed.all_gather(metrics_gathered, metrics[name])
 
+                if rank==0:
+                    gathered_len = len(metrics_gathered) if not is_last else len(metrics_gathered) - drop_number
+                    for i in range(gathered_len):
+                        if is_scalar[name] or not is_last:
+                            metrics_total[name].append(metrics_gathered[i].data.cpu())
+                        else:
+                            metrics_total[name].append(metrics_gathered[i][:last_shape[i]].data.cpu())
     # Move model back to the train mode.
     model.train()
 
     total_lm_loss /= eval_iters
-    metrics_avg = {key: value / eval_iters for key, value in metrics_total.items()}
-    for name in metrics_avg:
-        torch.distributed.all_reduce(metrics_avg[name].data)
-        metrics_avg[name].data /= args.world_size # model parallel will also reduce
-    return total_lm_loss, metrics_avg
+    # metrics_avg = {key: value / eval_iters for key, value in metrics_total.items()}
+    if rank==0:
+        for name in metrics_total:
+            if is_scalar[name]:
+                metrics_total[name] = torch.stack(metrics_total[name], dim=0)
+            else:
+                metrics_total[name] = torch.concat(metrics_total[name], dim=0)
+        if hooks['handle_metrics'] is not None:
+            metrics = hooks['handle_metrics'](metrics_total)
+        else:
+            for name in metrics_total:
+                assert is_scalar[name], 'you must return scalar metrics or implement handle_metrics hooks'
+            metrics = {key: sum(value.split(1,0))/len(value) for key, value in metrics_total.items()}
+    else:
+        metrics = None
+    return total_lm_loss, metrics
 
 def evaluate_and_print_results(prefix, data_iterator, model, eval_iters,
-                            args, timers, verbose=False, step=None, summary_writer=None, hooks={}):
+                            args, timers, has_last, split, verbose=False, step=None, summary_writer=None, hooks={}):
     """Helper function to evaluate and dump results on screen."""
-    # import line_profiler
-    # profile = line_profiler.LineProfiler(model.module.module.transformer.layers[0].forward)
-    # profile.enable()
-    # torch.cuda.empty_cache()
-    lm_loss, metrics = evaluate(data_iterator, model, eval_iters, args, timers, verbose, hooks=hooks)
-    # profile.disable()
-    # import sys
-    # profile.print_stats(sys.stdout)
+    lm_loss, metrics = evaluate(data_iterator, model, eval_iters, args, timers, split, verbose, has_last, hooks=hooks)
     lm_ppl = math.exp(min(20, lm_loss))
-    report_evaluate_metrics(summary_writer, prefix, lm_loss, lm_ppl, step, metrics)
-
+    if torch.distributed.get_rank(group=mpu.get_data_parallel_group())==0:
+        report_evaluate_metrics(summary_writer, prefix, lm_loss, lm_ppl, step, metrics)
     return lm_loss
 
 
@@ -494,7 +539,7 @@ def report_evaluate_metrics(summary_writer, prefix, loss, ppl, step, avg_metrics
     string += 'loss: {:.6E} | '.format(loss)
     string += 'PPL: {:.6E}'.format(ppl)
     for key in avg_metrics:
-        string += ' {} {:.6E} |'.format(key, avg_metrics[key])
+        string += ' {} {:.6E} |'.format(key, avg_metrics[key].item())
     length = len(string) + 1
     print_rank_0('-' * 100)
     print_rank_0('-' * length)
@@ -505,66 +550,4 @@ def report_evaluate_metrics(summary_writer, prefix, loss, ppl, step, avg_metrics
         summary_writer.add_scalar(f'Train/valid_loss', loss, step)
         for key in avg_metrics:
             summary_writer.add_scalar('Train/valid_'+key, avg_metrics[key], step)
-        
-
-'''
-    Optional DeepSpeed Activation Checkpointing features
-    Gives access to partition activations, contiguous memory optimizations
-    and cpu checkpointing.
-
-    Activation checkpoint requires keep track of the random states
-    and setting the random seed for each MP process. Megatron uses
-    mpu.get_cuda_rng_tracker and mpu.model_parallel_cuda_manual_seed
-    for keeping track of the random states and setting the random seeds.
-    Since they are used in places outside of activation checkpointing,
-    we overwrite them to maintain consistency.
-
-    This must be done before all the calls to mpu.model_parallel_cuda_manual_seed
-    '''
-
-
-def set_deepspeed_activation_checkpointing(args):
-    deepspeed.checkpointing.configure(mpu, deepspeed_config=args.deepspeed_config, num_checkpoints=args.num_layers)
-    mpu.checkpoint = deepspeed.checkpointing.checkpoint
-    mpu.get_cuda_rng_tracker = deepspeed.checkpointing.get_cuda_rng_tracker
-    mpu.model_parallel_cuda_manual_seed = deepspeed.checkpointing.model_parallel_cuda_manual_seed
-
-
-def initialize_distributed(args):
-    """Initialize torch.distributed."""
-
-    # the automatic assignment of devices has been moved to arguments.py 
-    torch.cuda.set_device(args.device)
-    # Call the init process
-    init_method = 'tcp://'
-    args.master_ip = os.getenv('MASTER_ADDR', 'localhost')
-    args.master_port = os.getenv('MASTER_PORT', '6000')
-    init_method += args.master_ip + ':' + args.master_port
-    torch.distributed.init_process_group(
-        backend=args.distributed_backend,
-        world_size=args.world_size, rank=args.rank,
-        init_method=init_method)
-
-    # Set the model-parallel / data-parallel communicators.
-    mpu.initialize_model_parallel(args.model_parallel_size)
-
-    # Optional DeepSpeed Activation Checkpointing Features
-    if hasattr(args, "deepspeed") and args.deepspeed and args.deepspeed_activation_checkpointing:
-        set_deepspeed_activation_checkpointing(args)  # TODO manual model-parallel seed
-    else:
-        mpu.get_cuda_rng_tracker = None
-
-def set_random_seed(seed):
-    """Set random seed for reproducability."""
-    if seed is not None and seed > 0:
-        random.seed(seed)
-        np.random.seed(seed)
-        torch.manual_seed(seed)
-        torch.cuda.manual_seed(seed)
-        torch.cuda.manual_seed_all(seed)  # if you are using multi-GPU.
-        torch.backends.cudnn.benchmark = False
-        torch.backends.cudnn.deterministic = True
-        torch.backends.cudnn.enabled = False
-        torch.backends.cuda.matmul.allow_tf32 = False
-        if hasattr(mpu, 'model_parallel_cuda_manual_seed'):
-            mpu.model_parallel_cuda_manual_seed(seed)
+        
\ No newline at end of file
